package de.mdelab.sdm.interpreter.core.patternmatcher.patternPartBased;

import java.util.LinkedHashSet;
import java.util.Set;

import de.mdelab.sdm.interpreter.core.SDMException;
import de.mdelab.sdm.interpreter.core.facade.MetamodelFacadeFactory;

/**
 * Subclass of the default matching strategy. This strategy maintains a log of
 * the matching order. This is required for for-each nodes. Here, the matching
 * order must be the same in each iteration to ensure that all matches are
 * returned and each match is returned only once.
 * 
 * @author Stephan Hildebrandt
 * 
 * @param <StoryPattern>
 * @param <StoryPatternObject>
 * @param <StoryPatternLink>
 * @param <Classifier>
 * @param <Feature>
 * @param <Expression>
 */
public class DefaultMatchingStrategyWithLog<StoryPattern, StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> extends
		DefaultMatchingStrategy<StoryPattern, StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression>
{

	private final LinkedHashSet<PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression>>	log;

	public DefaultMatchingStrategyWithLog(
			MetamodelFacadeFactory<?, ?, ?, StoryPattern, StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> facadeFactory)
	{
		super(facadeFactory);

		this.log = new LinkedHashSet<PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression>>();
	}

	@Override
	public PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> getNextPatternPartForMatching( 	final Set<PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression>> uncheckedPatternParts,
																																final PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> p_previouslyMatchedPatternPart,
																																final boolean p_previousMatchFound )
	throws SDMException {
//		if ( !p_previousMatchFound ) {
//			// DB: No need to re-evaluate the mAtching costs. We just need the last pattern part that was matched and that has been rolled back to unchecked
//			for ( final PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> part : uncheckedPatternParts ) {
//				if ( log.contains( part ) && part.matchingPossible() ) {
//					return part;
//				}
//			}
//		}
//		
//		
		PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression> patternPart = super.getNextPatternPartForMatching( uncheckedPatternParts, p_previouslyMatchedPatternPart, p_previousMatchFound );

		/*
		 * MB: This does not necessarily reflect the matching order which leads to the first match for the
		 * for-each node. This means that the structure of the pattern matcher's matching stack at the start
		 * of the second iteration of the for-each node may not correspond to the matching order recorded here.
		 * If this is the case, duplicate matches will be returned by the matcher if the log is actually used.
		 */
		if (patternPart != null && !this.log.contains(patternPart))
		{
			this.log.add(patternPart);
		}

		return patternPart;
	}

	public LinkedHashSet<PatternPart<StoryPatternObject, StoryPatternLink, Classifier, Feature, Expression>> getLog()
	{
		return this.log;
	}
}
